import { BrowserWindow } from 'electron';
import { WindowState } from '@ucap-webmessenger/native';
import { Channel } from '@ucap-webmessenger/native-electron';

export function getWindowState(window: Electron.BrowserWindow): WindowState {
  if (window.isFullScreen()) {
    return WindowState.FullScreen;
  } else if (window.isMaximized()) {
    return WindowState.Maximized;
  } else if (window.isMinimized()) {
    return WindowState.Minimized;
  } else if (!window.isVisible()) {
    return WindowState.Hidden;
  } else {
    return WindowState.Normal;
  }
}

export function registerWindowStateChangedEvents(window: BrowserWindow) {
  window.on('enter-full-screen', () =>
    sendWindowStateEvent(window, WindowState.FullScreen)
  );

  window.on('leave-full-screen', () =>
    sendWindowStateEvent(window, WindowState.Normal)
  );

  window.on('maximize', () =>
    sendWindowStateEvent(window, WindowState.Maximized)
  );
  window.on('minimize', () =>
    sendWindowStateEvent(window, WindowState.Minimized)
  );
  window.on('unmaximize', () =>
    sendWindowStateEvent(window, WindowState.Normal)
  );
  window.on('restore', () => sendWindowStateEvent(window, WindowState.Normal));
  window.on('hide', () => sendWindowStateEvent(window, WindowState.Hidden));
  window.on('show', () => {
    // because the app can be maximized before being closed - which will restore it
    // maximized on the next launch - this function should inspect the current state
    // rather than always assume it is a 'normal' launch
    sendWindowStateEvent(window, getWindowState(window));
  });
}

function sendWindowStateEvent(window: BrowserWindow, windowState: WindowState) {
  window.webContents.send(Channel.windowStateChanged, windowState);
}
