import {
  EventType,
  EventJson,
  FileEventJson,
  MassTextEventJson
} from '@ucap-webmessenger/protocol-event';
import { FileType } from '@ucap-webmessenger/protocol-file';

export class StringUtil {
  public static convertFinalEventMessage(
    eventType: EventType,
    finalEventMessage: EventJson
  ): string | null {
    let eventMessage: string = null;

    switch (eventType) {
      case EventType.Join:
      case EventType.Exit:
      case EventType.RenameRoom:
      case EventType.NotificationForTimerRoom:
      case EventType.GuideForRoomTimerChanged:
        {
          /**
           * 해당 타입은 메시지를 갱신하지 않는다.
           * @description Edit with ui-chat > messages.component.ts
           */
        }
        break;
      case EventType.Sticker:
        eventMessage = '스티커';
        break;
      case EventType.File:
        {
          const m = finalEventMessage as FileEventJson;

          if (FileType.Image === m.fileType) {
            eventMessage = '이미지';
          } else {
            eventMessage = '첨부파일';
          }
        }
        break;
      case EventType.VideoConference:
        eventMessage = '화상회의';
        break;
      case EventType.MassText:
        {
          const m = finalEventMessage as MassTextEventJson;
          eventMessage = m.content;
        }
        break;

      default:
        {
          const m = finalEventMessage as string;
          eventMessage = m;
        }

        break;
    }
    return eventMessage;
  }
}
